// Stochastic Tree Generator Sketch with MIDI output - 4 layers!
// This example was coded by Matt Boralnd for the UWTMC workshop on Oct 22, 2020
// Check out uwtmc.com for more info

// Use this parameter to select which MIDI output bus you use
// see the console in your web broswer for a list of available ports
// if it's not working, this is likely the reason
var midiSelect = 0;

var noteDur = 200;  //MIDI Note On message length (ms)
let channel = 1;    //MIDI Channel (1-16)

// Velocity Range that is picked from for each note
let velLow = 0.25;
let velHigh = 1;

// Probability Range that is picked from for each note
let probLow = 0.25;
let probHigh = 0.75;

// Note Lengths - these are the possible note lengths that are picked from for each note
// You can put in as many as you want!
let noteLengths = [0.25,0.5,0.75,1.0];

// This parameter controls the clock and how fast the sequencer runs
// Smaller numbers mean a faster tempo
let dt = 30;

// Fundamental - this is the root note for the scales and the sequence
let fundamental = 60;

// Possible Scale degrees
let possibleNotes = [];

// Scales in midi note intervals: fundamental + #
// Each scale is 8 values long, because that is used later when creating sets of notes
// Copy and paste different 8 note sets to change the scale notes are selected from
let scale = [0, 3, 5, 7,10,12,15,17];

// dorian           = [0, 2, 3, 5, 7, 9,10,12];
// phrygian         = [0, 1, 3, 5, 7, 8,10,12];
// lydian           = [0, 2, 4, 6, 7, 9,11,12];
// mixolydian       = [0, 2, 4, 5, 7, 9,10,12];
// aeolian          = [0, 2, 3, 5, 7, 8,10,12];
// locrian          = [0, 1, 3, 5, 6, 8,10,12];
// minor_pentatonic = [0, 3, 5, 7,10,12,15,17];
// major_pentatonic = [0, 2, 4, 7, 9,12,14,16];
// minor_blues      = [0, 3, 5, 6, 7,10,12,15];
// major_blues      = [0, 2, 3, 4, 7, 9,12,14];

// Note Variables
var n1; var v1; var p11; var l11; var l12;
var n11; var v11; var p111; var l111; var l112;
var n12; var v12; var p121; var l121; var l122;
var n111; var v111; var p1111; var l1111; var l1112;
var n112; var v112; var p1121; var l1121; var l1122;
var n121; var v121; var p1211; var l1211; var l1212;
var n122; var v122; var p1221; var l1221; var l1222;
var n1111; var v1111; var p11111; var l11111; var l11112;
var n1112; var v1112; var p11121; var l11121; var l11122;
var n1121; var v1121; var p11211; var l11211; var l11212;
var n1122; var v1122; var p11221; var l11221; var l11222;
var n1211; var v1211; var p12111; var l12111; var l12112;
var n1212; var v1212; var p12121; var l12121; var l12122;
var n1221; var v1221; var p12211; var l12211; var l12212;
var n1222; var v1222; var p12221; var l12221; var l12222;

// Location Variables
var x1; var y1; var x11; var y11;
var x12; var y12; var x111; var y111;
var x112; var y112; var x121; var y121;
var x122; var y122; var x1111; var y1111;
var x1112; var y1112; var x1121; var y1121;
var x1122; var y1122; var x1211; var y1211;
var x1212; var y1212; var x1221; var y1221;
var x1222; var y1222; var x11111; var y11111;
var x11112; var y11112; var x11121; var y11121;
var x11122; var y11122; var x11211; var y11211;
var x11212; var y11212; var x11221; var y11221;
var x11222; var y11222; var x12111; var y12111;
var x12112; var y12112; var x12121; var y12121;
var x12122; var y12122; var x12211; var y12211;
var x12212; var y12212; var x12221; var y12221;
var x12222; var y12222;

// Create a variable to be used for MIDI output
let midiOutput;

// Create a button variable to be used later
let button;

// These are General Parameters used in the sequencer
let scl = 120;   // Scale of line length
let scn = 75;   // Scale of node diameter
let segNum = 0; // Current segment

// MIDI variables
var noteNum;   //MIDI Note Number (0-127)
var noteVel;  //MIDI Note Velocity (0-1)

// Path Variables used to store the path of the sequencer
let notePath = [0,0,0,0];
let velPath = [0,0,0,0];
let timePath = [0,0,0,0];
let branchPath = [0,0,0,0];
let clock = 0;

// The setup function is run once when the page is loaded
function setup() {
  createCanvas(900, 500);
  background(210);
  fill(0);
  calcNotes();
  initParameters();
  calcLocations();
  textSize(24);
  setInterval(metro, dt); // This calls the metro function repeatedly after dt milliseconds
  smooth();
  button = createButton('Update');
  button.position(19, 19);
  button.mousePressed(updateMap);
}

// The Draw function loops over and over
function draw() {
  drawBeat();
}

// This function updates the map after the UPDATE button is clicked
function updateMap(){
  initParameters();
  calcLocations();
  calcPath();
  drawBranches();
}

// This function calculates the set of possible notes selected from at random
function calcNotes(){
  for(let n = 0; n < 8; n++){
    possibleNotes[n] = scale[n] - 12;
  }
  for(let n = 8; n < 15; n++){
    possibleNotes[n] = scale[n-7];
  }
}

// Calculate the locations of each node in the map - we'll draw them later
function calcLocations(){
  // n1 Location
  x1 = width/2;
  y1 = 50;

  // n11 Location
  x11 = x1+scl*l11*cos(2/3*PI);
  y11 = y1+scl*l11*sin(2/3*PI);
  // n12 Location
  x12 = x1+scl*l12*cos(1/3*PI);
  y12 = y1+scl*l12*sin(1/3*PI);

  // n111 Location
  x111 = x11+scl*l111*cos(0.75*PI);
  y111 = y11+scl*l111*sin(0.75*PI);
  // n112 Location
  x112 = x11+scl*l112*cos(0.5*PI);
  y112 = y11+scl*l112*sin(0.5*PI);
  // n121 Location
  x121 = x12+scl*l121*cos(0.5*PI);
  y121 = y12+scl*l121*sin(0.5*PI);
  // n122 Location
  x122 = x12+scl*l122*cos(0.25*PI);
  y122 = y12+scl*l122*sin(0.25*PI);

  // n1111 Location
  x1111 = x111+scl*l1111*cos(1.25*PI);
  y1111 = y111+scl*l1111*sin(1.25*PI);
  // n1112 Location
  x1112 = x111+scl*l1112*cos(PI);
  y1112 = y111+scl*l1112*sin(PI);
  // n1121 Location
  x1121 = x112+scl*l1121*cos(2/3*PI);
  y1121 = y112+scl*l1121*sin(2/3*PI);
  // n1122 Location
  x1122 = x112+scl*l1122*cos(0.5*PI);
  y1122 = y112+scl*l1122*sin(0.5*PI);
  // n1211 Location
  x1211 = x121+scl*l1211*cos(0.5*PI);
  y1211 = y121+scl*l1211*sin(0.5*PI);
  // n1212 Location
  x1212 = x121+scl*l1212*cos(1/3*PI);
  y1212 = y121+scl*l1212*sin(1/3*PI);
  // n1221 Location
  x1221 = x122+scl*l1221*cos(0*PI);
  y1221 = y122+scl*l1221*sin(0*PI);
  // n1222 Location
  x1222 = x122+scl*l1222*cos(-0.25*PI);
  y1222 = y122+scl*l1222*sin(-0.25*PI);

  // n11111 Location
  x11111 = x1111+scl*l11111*cos(1.25*PI);
  y11111 = y1111+scl*l11111*sin(1.25*PI);
  // n11112 Location
  x11112 = x1111+scl*l11112*cos(PI);
  y11112 = y1111+scl*l11112*sin(PI);
  // n11121 Location
  x11121 = x1112+scl*l11121*cos(PI);
  y11121 = y1112+scl*l11121*sin(PI);
  // n11122 Location
  x11122 = x1112+scl*l11122*cos(2/3*PI);
  y11122 = y1112+scl*l11122*sin(2/3*PI);
  // n11211 Location
  x11211 = x1121+scl*l11211*cos(PI);
  y11211 = y1121+scl*l11211*sin(PI);
  // n11212 Location
  x11212 = x1121+scl*l11212*cos(2/3*PI);
  y11212 = y1121+scl*l11212*sin(2/3*PI);

  // n11221 Location
  x11221 = x1122+scl*l11221*cos(2/3*PI);
  y11221 = y1122+scl*l11221*sin(2/3*PI);
  // n11222 Location
  x11222 = x1122+scl*l11222*cos(0.5*PI);
  y11222 = y1122+scl*l11222*sin(0.5*PI);
  // n12111 Location
  x12111 = x1211+scl*l12111*cos(0.5*PI);
  y12111 = y1211+scl*l12111*sin(0.5*PI);
  // n12112 Location
  x12112 = x1211+scl*l12112*cos(1/3*PI);
  y12112 = y1211+scl*l12112*sin(1/3*PI);
  // n12121 Location
  x12121 = x1212+scl*l12121*cos(1/3*PI);
  y12121 = y1212+scl*l12121*sin(1/3*PI);
  // n12122 Location
  x12122 = x1212+scl*l12122*cos(0);
  y12122 = y1212+scl*l12122*sin(0);
  // n12211 Location
  x12211 = x1221+scl*l12211*cos(1/3*PI);
  y12211 = y1221+scl*l12211*sin(1/3*PI);
  // n12212 Location
  x12212 = x1221+scl*l12212*cos(0);
  y12212 = y1221+scl*l12212*sin(0);
  // n12221 Location
  x12221 = x1222+scl*l12221*cos(0);
  y12221 = y1222+scl*l12221*sin(0);
  // n12222 Location
  x12222 = x1222+scl*l12222*cos(-0.25*PI);
  y12222 = y1222+scl*l12222*sin(-0.25*PI);
}

// Initialize the many musical parameters of the map
// Each node needs a pitch, velocity, probability and path lengths
// There are lots of nodes, so this gets long
function initParameters(){
  // layer 1
  // Branch 0,1
  n1 = fundamental;     // MIDI note number 0-127
  v1 = 1.0;      // Velocity Value from 0-1
  p11 = random(probLow,probHigh);   // Probability of taking this branch
  l11 = random(noteLengths);   // Path 1 Note Length in beat divisions
  l12 = random(noteLengths);   // Path 2 Note Length in beat divisions

  // layer 2
  // Branch  2,3
  n11 = random(possibleNotes)+fundamental;
  v11 = random(velLow,velHigh);
  p111 = random(probLow,probHigh);
  l111 = random(noteLengths);
  l112 = random(noteLengths);

  // Branch 4,5
  n12 = random(possibleNotes)+fundamental;
  v12 = random(velLow,velHigh);
  p121 = random(probLow,probHigh);
  l121 = random(noteLengths);
  l122 = random(noteLengths);

  // layer 3
  // Branch 6,7
  n111 = random(possibleNotes)+fundamental;
  v111 = random(velLow,velHigh);
  p1111 = random(probLow,probHigh);
  l1111 = random(noteLengths);
  l1112 = random(noteLengths);

  // Branch 8,9
  n112 = random(possibleNotes)+fundamental;
  v112 = random(velLow,velHigh);
  p1121 = random(probLow,probHigh);
  l1121 = random(noteLengths);
  l1122 = random(noteLengths);

  // Branch 10,11
  n121 = random(possibleNotes)+fundamental;
  v121 = random(velLow,velHigh);
  p1211 = random(probLow,probHigh);
  l1211 = random(noteLengths);
  l1212 = random(noteLengths);

  // Branch 12,13
  n122 = random(possibleNotes)+fundamental;
  v122 = random(velLow,velHigh);
  p1221 = random(probLow,probHigh);
  l1221 = random(noteLengths);
  l1222 = random(noteLengths);

  // layer 4
  // Branch 14,15
  n1111 = random(possibleNotes)+fundamental;
  v1111 = random(velLow,velHigh);
  p11111 = random(probLow,probHigh);
  l11111 = random(noteLengths);
  l11112 = random(noteLengths);

  // Branch 16,17
  n1112 = random(possibleNotes)+fundamental;
  v1112 = random(velLow,velHigh);
  p11121 = random(probLow,probHigh);
  l11121= random(noteLengths);
  l11122 = random(noteLengths);

  // Branch 18,19
  n1121 = random(possibleNotes)+fundamental;
  v1121 = random(velLow,velHigh);
  p11211 = random(probLow,probHigh);
  l11211 = random(noteLengths);
  l11212 = random(noteLengths);

  // Branch 20,21
  n1122 = random(possibleNotes)+fundamental;
  v1122 = random(velLow,velHigh);
  p11221 = random(probLow,probHigh);
  l11221 = random(noteLengths);
  l11222 = random(noteLengths);

  // Branch 22,23
  n1211 = random(possibleNotes)+fundamental;
  v1211 = random(velLow,velHigh);
  p12111 = random(probLow,probHigh);
  l12111 = random(noteLengths);
  l12112 = random(noteLengths);

  // Branch 24,25
  n1212 = random(possibleNotes)+fundamental;
  v1212 = random(velLow,velHigh);
  p12121 = random(probLow,probHigh);
  l12121 = random(noteLengths);
  l12122 = random(noteLengths);

  // Branch 26,27
  n1221 = random(possibleNotes)+fundamental;
  v1221 = random(velLow,velHigh);
  p12211 = random(probLow,probHigh);
  l12211 = random(noteLengths);
  l12212 = random(noteLengths);

  // Branch 28,29
  n1222 = random(possibleNotes)+fundamental;
  v1222 = random(velLow,velHigh);
  p12221 = random(probLow,probHigh);
  l12221 = random(noteLengths);
  l12222 = random(noteLengths);
}

// This function acts as the clock - sending MIDI notes at the appropriate times
// It also sends MIDI clock, so you could synchronize other instruments, too
function metro(){
  midiOutput.sendClock();
  if (clock < 24*timePath[0]){
    clock = clock + 1;
    if(clock >= 24*timePath[0]){
      sendMIDINote(notePath[1],velPath[1]);
    }
  } else if (clock < (24*timePath[0]+24*timePath[1])){
    clock = clock + 1;
    if(clock >= (24*timePath[0]+24*timePath[1])){
      sendMIDINote(notePath[2],velPath[2]);
    }
  } else if (clock < (24*timePath[0]+24*timePath[1]+24*timePath[2])){
    clock = clock + 1;
    if(clock >= (24*timePath[0]+24*timePath[1]+24*timePath[2])){
      sendMIDINote(notePath[3],velPath[3]);
    }
  } else if (clock < (24*timePath[0]+24*timePath[1]+24*timePath[2]+24*timePath[3])){
    clock = clock + 1;
  } else {
    clock = 0;
    sendMIDINote(notePath[0],velPath[0]);
    calcPath();
    drawBranches();
  }
}

// This functio draws the current clock tick and beat location onto the map
// These show up as the little white dots
function drawBeat(){
  let xpos = 0;
  let ypos = 0;
// Layer 1
  if(clock < 24*timePath[0]){
    if(branchPath[0] == 0){
      xpos = x1+clock/(24*timePath[0])*(x11-x1);
      ypos = y1+clock/(24*timePath[0])*(y11-y1);
    } else {
      xpos = x1+clock/(24*timePath[0])*(x12-x1);
      ypos = y1+clock/(24*timePath[0])*(y12-y1);
    }
// Layer 2
} else if (clock < (24*timePath[0]+24*timePath[1])){
    if(branchPath[1] == 2){
      xpos = x11+(clock-24*timePath[0])/(24*timePath[1])*(x111-x11);
      ypos = y11+(clock-24*timePath[0])/(24*timePath[1])*(y111-y11);
    } else if(branchPath[1] == 3){
      xpos = x11+(clock-24*timePath[0])/(24*timePath[1])*(x112-x11);
      ypos = y11+(clock-24*timePath[0])/(24*timePath[1])*(y112-y11);
    } else if(branchPath[1] == 4){
      xpos = x12+(clock-24*timePath[0])/(24*timePath[1])*(x121-x12);
      ypos = y12+(clock-24*timePath[0])/(24*timePath[1])*(y121-y12);
    } else {
      xpos = x12+(clock-24*timePath[0])/(24*timePath[1])*(x122-x12);
      ypos = y12+(clock-24*timePath[0])/(24*timePath[1])*(y122-y12);
    }
// Layer 3
} else if(clock < (24*timePath[0]+24*timePath[1])+24*timePath[2]) {
      if(branchPath[2] == 6){
        xpos = x111+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(x1111-x111);
        ypos = y111+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(y1111-y111);
      } else if(branchPath[2] == 7){
        xpos = x111+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(x1112-x111);
        ypos = y111+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(y1112-y111);
      } else if(branchPath[2] == 8){
        xpos = x112+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(x1121-x112);
        ypos = y112+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(y1121-y112);
      } else if(branchPath[2] == 9){
        xpos = x112+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(x1122-x112);
        ypos = y112+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(y1122-y112);
      } else if(branchPath[2] == 10){
        xpos = x121+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(x1211-x121);
        ypos = y121+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(y1211-y121);
      } else if(branchPath[2] == 11){
        xpos = x121+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(x1212-x121);
        ypos = y121+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(y1212-y121);
      } else if(branchPath[2] == 12){
        xpos = x122+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(x1221-x122);
        ypos = y122+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(y1221-y122);
      } else {
        xpos = x122+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(x1222-x122);
        ypos = y122+(clock-24*(timePath[0]+timePath[1]))/(24*timePath[2])*(y1222-y122);
      }
// Layer 4
} else {
  if(branchPath[3] == 14){
    xpos = x1111+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x11111-x1111);
    ypos = y1111+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y11111-y1111);
  } else if(branchPath[3] == 15){
    xpos = x1111+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x11112-x1111);
    ypos = y1111+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y11112-y1111);
  } else if(branchPath[3] == 16){
    xpos = x1112+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x11121-x1112);
    ypos = y1112+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y11121-y1112);
  } else if(branchPath[3] == 17){
    xpos = x1112+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x11122-x1112);
    ypos = y1112+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y11122-y1112);
  } else if(branchPath[3] == 18){
    xpos = x1121+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x11211-x1121);
    ypos = y1121+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y11211-y1121);
  } else if(branchPath[3] == 19){
    xpos = x1121+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x11212-x1121);
    ypos = y1121+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y11212-y1121);
  } else if(branchPath[3] == 20){
    xpos = x1122+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x11221-x1122);
    ypos = y1122+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y11221-y1122);
  } else if(branchPath[3] == 21){
    xpos = x1122+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x11222-x1122);
    ypos = y1122+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y11222-y1122);
  } else if(branchPath[3] == 22){
    xpos = x1211+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x12111-x1211);
    ypos = y1211+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y12111-y1211);
  } else if(branchPath[3] == 23){
    xpos = x1211+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x12112-x1211);
    ypos = y1211+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y12112-y1211);
  } else if(branchPath[3] == 24){
    xpos = x1212+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x12121-x1212);
    ypos = y1212+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y12121-y1212);
  } else if(branchPath[3] == 25){
    xpos = x1212+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x12122-x1212);
    ypos = y1212+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y12122-y1212);
  } else if(branchPath[3] == 26){
    xpos = x1221+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x12211-x1221);
    ypos = y1221+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y12211-y1221);
  } else if(branchPath[3] == 27){
    xpos = x1221+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x12212-x1221);
    ypos = y1221+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y12212-y1221);
  } else if(branchPath[3] == 28){
    xpos = x1222+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x12221-x1222);
    ypos = y1222+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y12221-y1222);
  } else {
    xpos = x1222+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(x12222-x1222);
    ypos = y1222+(clock-24*(timePath[0]+timePath[1]+timePath[2]))/(24*timePath[3])*(y12222-y1222);
  }
}
  fill(255);
  ellipse(xpos,ypos,scl*0.05,scl*0.05);
}

// Each time the end of the path is reached we calculate the next path for the next cycle
function calcPath(){
  notePath[0] = n1;
  velPath[0] = v1;
  // 0
  if(random()<p11){
    branchPath[0] = 0;
    timePath[0] = l11;
    notePath[1] = n11;
    velPath[1] = v11;
    //2
    if(random()<p111){
      branchPath[1] = 2;
      timePath[1] = l111;
      notePath[2] = n111;
      velPath[2] = v111;
      //6
      if(random()<p1111){
        branchPath[2] = 6;
        timePath[2] = l1111;
        notePath[3] = n1111;
        velPath[3] = v1111;
        //14
        if(random()<p11111){
          branchPath[3] = 14;
          timePath[3] = l11111;
        //15
        } else {
          branchPath[3] = 15;
          timePath[3] = l11112;
        }
      //7
      } else {
        branchPath[2] = 7;
        timePath[2] = l1112;
        notePath[3] = n1112;
        velPath[3] = v1112;
        //16
        if(random()<p11121){
          branchPath[3] = 16;
          timePath[3] = l11121;
        //17
        } else {
          branchPath[3] = 17;
          timePath[3] = l11122;
        }
      }
    //3
    } else {
      branchPath[1] = 3;
      timePath[1] = l112;
      notePath[2] = n112;
      velPath[2] = v112;
      //8
      if(random()<p1121){
        branchPath[2] = 8;
        timePath[2] = l1121;
        notePath[3] = n1121;
        velPath[3] = v1121;
        //18
        if(random()<p11211){
          branchPath[3] = 18;
          timePath[3] = l11211;
        //19
        } else {
          branchPath[3] = 19;
          timePath[3] = l11212;
        }
      //9
      } else {
        branchPath[2] = 9;
        timePath[2] = l1122;
        notePath[3] = n1122;
        velPath[3] = v1122;
        //20
        if(random()<p11221){
          branchPath[3] = 20;
          timePath[3] = l11221;
        //21
        } else {
          branchPath[3] = 21;
          timePath[3] = l11222;
        }
      }
    }
  // 1
  } else {
    branchPath[0] = 1;
    timePath[0] = l12;
    notePath[1] = n12;
    velPath[1] = v12;
    //4
    if(random()<p121){
      branchPath[1] = 4;
      timePath[1] = l121;
      notePath[2] = n121;
      velPath[2] = v121;
      //10
      if(random()<p1211){
        branchPath[2] = 10;
        timePath[2] = l1211;
        notePath[3] = n1211;
        velPath[3] = v1211;
        //22
        if(random()<p12111){
          branchPath[3] = 22;
          timePath[3] = l12111;
        //23
        } else {
          branchPath[3] = 23;
          timePath[3] = l12112;
        }
      //11
      } else {
        branchPath[2] = 11;
        timePath[2] = l1212;
        notePath[3] = n1212;
        velPath[3] = v1212;
        //24
        if(random()<p12121){
          branchPath[3] = 24;
          timePath[3] = l12121;
        //25
        } else {
          branchPath[3] = 25;
          timePath[3] = l12122;
        }
      }
    //5
    } else {
      branchPath[1] = 5;
      timePath[1] = l122;
      notePath[2] = n122;
      velPath[2] = v122;
      //12
      if(random()<p1221){
        branchPath[2] = 12;
        timePath[2] = l1221;
        notePath[3] = n1221;
        velPath[3] = v1221;
        //26
        if(random()<p12211){
          branchPath[3] = 26;
          timePath[3] = l12211;
        //27
        } else {
          branchPath[3] = 27;
          timePath[3] = l12212;
        }
      //13
      } else {
        branchPath[2] = 13;
        timePath[2] = l1222;
        notePath[3] = n1222;
        velPath[3] = v1222;
        //28
        if(random()<p12221){
          branchPath[3] = 28;
          timePath[3] = l12221;
        //29
        } else {
          branchPath[3] = 29;
          timePath[3] = l12222;
        }
      }
    }
  }
}

// This ia a helper function that determines the colour mapping of the nodes
function calcColour(nn){
  let c = [127-+(nn-fundamental)*10,127+(nn-fundamental)*10,200];
  return c;
}

// This function draws the branches and the background map
function drawBranches(){
  background(210);
  fill(20);
  text("Using "+WebMidi.outputs[midiSelect].name,40,height-20);
//Layer 1
  stroke(120);
  strokeWeight(10*p11);
  line(x1,y1,x11,y11);
  strokeWeight(10*(1-p11));
  line(x1,y1,x12,y12);
  noStroke();
  fill(calcColour(n1));
  ellipse(x1, y1, scn*v1/2, scn*v1/2);

//Layer 2
  //Draw n11
  stroke(120);
  strokeWeight(10*p111);
  line(x11,y11,x111,y111);
  strokeWeight(10*(1-p111));
  line(x11,y11,x112,y112);
  noStroke();
  fill(calcColour(n11));
  ellipse(x11, y11, scn*v11/2, scn*v11/2);
  //Draw n12
  stroke(120);
  strokeWeight(10*p121);
  line(x12,y12,x121,y121);
  strokeWeight(10*(1-p121));
  line(x12,y12,x122,y122);
  noStroke();
  fill(calcColour(n12));
  ellipse(x12, y12, scn*v12/2, scn*v12/2);

//Layer 3
  //Draw n111
  stroke(120);
  strokeWeight(10*p1111);
  line(x111,y111,x1111,y1111);
  strokeWeight(10*(1-p1111));
  line(x111,y111,x1112,y1112);
  noStroke();
  fill(calcColour(n111));
  ellipse(x111, y111, scn*v111/2, scn*v111/2);
  //Draw n112
  stroke(120);
  strokeWeight(10*p1121);
  line(x112,y112,x1121,y1121);
  strokeWeight(10*(1-p1121));
  line(x112,y112,x1122,y1122);
  noStroke();
  fill(calcColour(n112));
  ellipse(x112, y112, scn*v112/2, scn*v112/2);
  //Draw n121
  stroke(120);
  strokeWeight(10*p1211);
  line(x121,y121,x1211,y1211);
  strokeWeight(10*(1-p1211));
  line(x121,y121,x1212,y1212);
  noStroke();
  fill(calcColour(n121));
  ellipse(x121, y121, scn*v121/2, scn*v121/2);
  //Draw n122
  stroke(120);
  strokeWeight(10*p1221);
  line(x122,y122,x1221,y1221);
  strokeWeight(10*(1-p1221));
  line(x122,y122,x1222,y1222);
  noStroke();
  fill(calcColour(n122));
  ellipse(x122, y122, scn*v122/2, scn*v122/2);

//Layer 4
  //Draw n1111
  stroke(120);
  strokeWeight(10*p11111);
  line(x1111,y1111,x11111,y11111);
  strokeWeight(10*(1-p1111));
  line(x1111,y1111,x11112,y11112);
  noStroke();
  fill(calcColour(n1111));
  ellipse(x1111, y1111, scn*v1111/2, scn*v1111/2);
  //Draw n1112
  stroke(120);
  strokeWeight(10*p11121);
  line(x1112,y1112,x11121,y11121);
  strokeWeight(10*(1-p11121));
  line(x1112,y1112,x11122,y11122);
  noStroke();
  fill(calcColour(n1112));
  ellipse(x1112, y1112, scn*v1112/2, scn*v1112/2);
  //Draw n1121
  stroke(120);
  strokeWeight(10*p11211);
  line(x1121,y1121,x11211,y11211);
  strokeWeight(10*(1-p11211));
  line(x1121,y1121,x11212,y11212);
  noStroke();
  fill(calcColour(n1121));
  ellipse(x1121, y1121, scn*v1121/2, scn*v1121/2);
  //Draw n1122
  stroke(120);
  strokeWeight(10*p11221);
  line(x1122,y1122,x11221,y11221);
  strokeWeight(10*(1-p11221));
  line(x1122,y1122,x11222,y11222);
  noStroke();
  fill(calcColour(n1122));
  ellipse(x1122, y1122, scn*v1122/2, scn*v1122/2);
  //Draw n1211
  stroke(120);
  strokeWeight(10*p12111);
  line(x1211,y1211,x12111,y12111);
  strokeWeight(10*(1-p12111));
  line(x1211,y1211,x12112,y12112);
  noStroke();
  fill(calcColour(n1211));
  ellipse(x1211, y1211, scn*v1211/2, scn*v1211/2);
  //Draw n1212
  stroke(120);
  strokeWeight(10*p12121);
  line(x1212,y1212,x12121,y12121);
  strokeWeight(10*(1-p12121));
  line(x1212,y1212,x12122,y12122);
  noStroke();
  fill(calcColour(n1212));
  ellipse(x1212, y1212, scn*v1212/2, scn*v1212/2);
  //Draw n1221
  stroke(120);
  strokeWeight(10*p12211);
  line(x1221,y1221,x12211,y12211);
  strokeWeight(10*(1-p12211));
  line(x1221,y1221,x12212,y12212);
  noStroke();
  fill(calcColour(n1221));
  ellipse(x1221, y1221, scn*v1221/2, scn*v1221/2);
  //Draw n1222
  stroke(120);
  strokeWeight(10*p12221);
  line(x1222,y1222,x12221,y12221);
  strokeWeight(10*(1-p12221));
  line(x1222,y1222,x12222,y12222);
  noStroke();
  fill(calcColour(n1222));
  ellipse(x1222, y1222, scn*v1222/2, scn*v1222/2);
}

// This function sends midi note data
function sendMIDINote(noteNum_, noteVel_){
  if(midiOutput){
		midiOutput.playNote(noteNum_, channel, {duration: noteDur, velocity: noteVel_});
	}
	else{
		console.log("Looks like there is no MIDI output device.");
	}
}

// This function enables the webMIDI components and starts the connection
// from your web browser to your MIDI bus
WebMidi.enable(function (err) {
  if (err) {
    console.log("WebMidi could not be enabled.", err);
  }

  // Print available MIDI outputs
  for(let i = 0; i < WebMidi.outputs.length; i++){
  	console.log(WebMidi.outputs[i].name);
  }

  // From the list on the console, pick an output name:
  midiOutput = WebMidi.getOutputByName(WebMidi.outputs[midiSelect].name);

});
